#
# bio/appl/fasta/format6.rb - FASTA output (-m 6) parser
# 
#   Copyright (C) 2002 KATAYAMA Toshiaki <k@bioruby.org>
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
#  $Id: format6.rb,v 1.1 2002/05/28 14:49:07 k Exp $
#

module Bio
  class Fasta

    class Report

      # will be ported from sample/{fasta2tab,ssearch2tab,tfastx2tab}.rb

      def initialize
	raise NotImplementedError
      end
    end

  end
end

