# 
# bio/data/keggorg.rb - KEGG organism code module
# 
#   Copyright (C) 2001, 2002  KATAYAMA Toshiaki <k@bioruby.org> 
#   Copyright (C) 2002, 2003  Masumi Itoh <m@bioruby.org> 
# 
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
#  $Id: keggorg.rb,v 0.13 2003/06/02 06:12:16 m Exp $
#

module Bio

  class KEGG

     ORG = {
    'aae' => [ 'A.aeolicus', 'Aquifex aeolicus' ],
    'afu' => [ 'A.fulgidus', 'Archaeoglobus fulgidus' ],
    'ana' => [ 'Anabaena', 'Anabaena sp.' ],
    'ape' => [ 'A.pernix', 'Aeropyrum pernix' ],
    'atc' => [ 'A.tumefaciens_C', 'Agrobacterium tumefaciens C58 Cereon' ],
    'ath' => [ 'A.thaliana', 'Arabidopsis thaliana' ],
    'atu' => [ 'A.tumefaciens', 'Agrobacterium tumefaciens C58 UWash' ],
    'bab' => [ 'B.aphidicola_Bp', 'Buchnera aphidicola Bp' ],
    'ban' => [ 'B.anthracis', 'Bacillus anthracis' ],
    'bas' => [ 'B.aphidicola_Sg', 'Buchnera aphidicola Sg' ],
    'bbu' => [ 'B.burgdorferi', 'Borrelia burgdorferi' ],
    'bce' => [ 'B.cereus', 'Bacillus cereus' ],
    'bha' => [ 'B.halodurans', 'Bacillus halodurans' ],
    'bja' => [ 'B.japonicum', 'Bradyrhizobium japonicum' ],
    'blo' => [ 'B.longum', 'Bifidobacterium longum' ],
    'bme' => [ 'B.melitensis', 'Brucella melitensis' ],
    'bms' => [ 'B.suis', 'Brucella suis' ],
    'bsu' => [ 'B.subtilis', 'Bacillus subtilis' ],
    'bth' => [ 'B.thetaiotaomicron', 'Bacteroides thetaiotaomicron' ],
    'buc' => [ 'Buchnera', 'Buchnera sp.' ],
    'cac' => [ 'C.acetobutylicum', 'Clostridium acetobutylicum' ],
    'cbu' => [ 'C.burnetii', 'Coxiella burnetii' ],
    'cca' => [ 'C.caviae', 'Chlamydophila caviae' ],
    'ccr' => [ 'C.crescentus', 'Caulobacter crescentus' ],
    'cef' => [ 'C.efficiens', 'Corynebacterium efficiens' ],
    'cel' => [ 'C.elegans', 'Caenorhabditis elegans' ],
    'cgl' => [ 'C.glutamicum', 'Corynebacterium glutamicum' ],
    'cje' => [ 'C.jejuni', 'Campylobacter jejuni' ],
    'cmu' => [ 'C.muridarum', 'Chlamydia muridarum' ],
    'cpa' => [ 'C.pneumoniae_AR39', 'Chlamydophila pneumoniae AR39' ],
    'cpe' => [ 'C.perfringens', 'Clostridium perfringens' ],
    'cpj' => [ 'C.pneumoniae_J138', 'Chlamydophila pneumoniae J138' ],
    'cpn' => [ 'C.pneumoniae', 'Chlamydophila pneumoniae CWL029' ],
    'ctc' => [ 'C.tetani', 'Clostridium tetani' ],
    'cte' => [ 'C.tepidum', 'Chlorobium tepidum' ],
    'ctr' => [ 'C.trachomatis', 'Chlamydia trachomatis' ],
    'ddi' => [ 'D.discoideum', 'Dictyostelium discoideum' ],
    'dme' => [ 'D.melanogaster', 'Drosophila melanogaster' ],
    'dra' => [ 'D.radiodurans', 'Deinococcus radiodurans' ],
    'dre' => [ 'D.rerio', 'Danio rerio' ],
    'ecc' => [ 'E.coli_CFT073', 'Escherichia coli CFT073' ],
    'ece' => [ 'E.coli_O157', 'Escherichia coli O157 EDL933' ],
    'ecj' => [ 'E.coli_J', 'Escherichia coli K-12 W3110' ],
    'eco' => [ 'E.coli', 'Escherichia coli K-12 MG1655' ],
    'ecs' => [ 'E.coli_O157J', 'Escherichia coli O157 Sakai' ],
    'ecu' => [ 'E.cuniculi', 'Encephalitozoon cuniculi' ],
    'efa' => [ 'E.faecalis', 'Enterococcus faecalis' ],
    'fnu' => [ 'F.nucleatum', 'Fusobacterium nucleatum' ],
    'hal' => [ 'Halobacterium', 'Halobacterium sp.' ],
    'hin' => [ 'H.influenzae', 'Haemophilus influenzae' ],
    'hpj' => [ 'H.pylori_J99', 'Helicobacter pylori J99' ],
    'hpy' => [ 'H.pylori', 'Helicobacter pylori 26695' ],
    'hsa' => [ 'H.sapiens', 'Homo sapiens' ],
    'lil' => [ 'L.interrogans', 'Leptospira interrogans' ],
    'lin' => [ 'L.innocua', 'Listeria innocua' ],
    'lla' => [ 'L.lactis', 'Lactococcus lactis' ],
    'lmo' => [ 'L.monocytogenes', 'Listeria monocytogenes' ],
    'lpl' => [ 'L.plantarum', 'Lactobacillus plantarum' ],
    'mac' => [ 'M.acetivorans', 'Methanosarcina acetivorans' ],
    'mge' => [ 'M.genitalium', 'Mycoplasma genitalium' ],
    'mja' => [ 'M.jannaschii', 'Methanococcus jannaschii' ],
    'mka' => [ 'M.kandleri', 'Methanopyrus kandleri' ],
    'mle' => [ 'M.leprae', 'Mycobacterium leprae' ],
    'mlo' => [ 'M.loti', 'Mesorhizobium loti' ],
    'mma' => [ 'M.mazei', 'Methanosarcina mazei' ],
    'mmu' => [ 'M.musculus', 'Mus musculus' ],
    'mpe' => [ 'M.penetrans', 'Mycoplasma penetrans' ],
    'mpn' => [ 'M.pneumoniae', 'Mycoplasma pneumoniae' ],
    'mpu' => [ 'M.pulmonis', 'Mycoplasma pulmonis' ],
    'mtc' => [ 'M.tuberculosis_CDC1551', 'Mycobacterium tuberculosis CDC1551' ],
    'mth' => [ 'M.thermoautotrophicum', 'Methanobacterium thermoautotrophicum' ],
    'mtu' => [ 'M.tuberculosis', 'Mycobacterium tuberculosis H37Rv' ],
    'neu' => [ 'N.europaea', 'Nitrosomonas europaea' ],
    'nma' => [ 'N.meningitidis_A', 'Neisseria meningitidis serogroup A' ],
    'nme' => [ 'N.meningitidis', 'Neisseria meningitidis serogroup B' ],
    'oih' => [ 'O.iheyensis', 'Oceanobacillus iheyensis' ],
    'osa' => [ 'O.sativa', 'Oryza sativa' ],
    'pab' => [ 'P.abyssi', 'Pyrococcus abyssi' ],
    'pae' => [ 'P.aeruginosa', 'Pseudomonas aeruginosa' ],
    'pai' => [ 'P.aerophilum', 'Pyrobaculum aerophilum ' ],
    'pfa' => [ 'P.falciparum', 'Plasmodium falciparum' ],
    'pfu' => [ 'P.furiosus', 'Pyrococcus furiosus' ],
    'pho' => [ 'P.horikoshii', 'Pyrococcus horikoshii' ],
    'pmu' => [ 'P.multocida', 'Pasteurella multocida' ],
    'ppu' => [ 'P.putida', 'Pseudomonas putida' ],
    'pst' => [ 'P.syringae', 'Pseudomonas syringae' ],
    'rco' => [ 'R.conorii', 'Rickettsia conorii' ],
    'rno' => [ 'R.norvegicus', 'Rattus norvegicus' ],
    'rpr' => [ 'R.prowazekii', 'Rickettsia prowazekii' ],
    'rso' => [ 'R.solanacearum', 'Ralstonia solanacearum' ],
    'sag' => [ 'S.agalactiae', 'Streptococcus agalactiae' ],
    'sam' => [ 'S.aureus_MW2', 'Staphylococcus aureus MW2' ],
    'san' => [ 'S.agalactiae_NEM316', 'Streptococcus agalactiae NEM316' ],
    'sau' => [ 'S.aureus_N315', 'Staphylococcus aureus N315' ],
    'sav' => [ 'S.aureus_Mu50', 'Staphylococcus aureus Mu50' ],
    'sce' => [ 'S.cerevisiae', 'Saccharomyces cerevisiae' ],
    'sco' => [ 'S.coelicolor', 'Streptomyces coelicolor' ],
    'sep' => [ 'S.epidermidis', 'Staphylococcus epidermidis' ],
    'sfl' => [ 'S.flexneri', 'Shigella flexneri' ],
    'sfx' => [ 'S.flexneri_2457T', 'Shigella flexneri 2457T' ],
    'sma' => [ 'S.avermitilis', 'Streptomyces avermitilis' ],
    'sme' => [ 'S.meliloti', 'Sinorhizobium meliloti' ],
    'smu' => [ 'S.mutans', 'Streptococcus mutans' ],
    'son' => [ 'S.oneidensis', 'Shewanella oneidensis' ],
    'spg' => [ 'S.pyogenes_M3', 'Streptococcus pyogenes M3' ],
    'spm' => [ 'S.pyogenes_M18', 'Streptococcus pyogenes M18' ],
    'spn' => [ 'S.pneumoniae', 'Streptococcus pneumoniae TIGR4' ],
    'spo' => [ 'S.pombe', 'Schizosaccharomyces pombe' ],
    'spr' => [ 'S.pneumoniae_R6', 'Streptococcus pneumoniae R6' ],
    'sps' => [ 'S.pyogenes_SSI1', 'Streptococcus pyogenes SSI1' ],
    'spy' => [ 'S.pyogenes', 'Streptococcus pyogenes' ],
    'sso' => [ 'S.solfataricus', 'Sulfolobus solfataricus' ],
    'stm' => [ 'S.typhimurium', 'Salmonella typhimurium' ],
    'sto' => [ 'S.tokodaii', 'Sulfolobus tokodaii' ],
    'stt' => [ 'S.typhi_Ty2', 'Salmonella typhi Ty2' ],
    'sty' => [ 'S.typhi', 'Salmonella typhi' ],
    'syn' => [ 'Synechocystis', 'Synechocystis sp.' ],
    'tac' => [ 'T.acidophilum', 'Thermoplasma acidophilum' ],
    'tel' => [ 'T.elongatus', 'Thermosynechococcus elongatus' ],
    'tma' => [ 'T.maritima', 'Thermotoga maritima' ],
    'tpa' => [ 'T.pallidum', 'Treponema pallidum' ],
    'tte' => [ 'T.tengcongensis', 'Thermoanaerobacter tengcongensis' ],
    'tvo' => [ 'T.volcanium', 'Thermoplasma volcanium' ],
    'twh' => [ 'T.whipplei', 'Tropheryma whipplei' ],
    'tws' => [ 'T.whipplei_S', 'Tropheryma whipplei TW08/27' ],
    'uur' => [ 'U.urealyticum', 'Ureaplasma urealyticum' ],
    'vch' => [ 'V.cholerae', 'Vibrio cholerae' ],
    'vpa' => [ 'V.parahaemolyticus', 'Vibrio parahaemolyticus' ],
    'vvu' => [ 'V.vulnificus', 'Vibrio vulnificus' ],
    'wbr' => [ 'W.brevipalpis', 'Wigglesworthia brevipalpis' ],
    'xac' => [ 'X.axonopodis', 'Xanthomonas axonopodis' ],
    'xcc' => [ 'X.campestris', 'Xanthomonas campestris' ],
    'xfa' => [ 'X.fastidiosa', 'Xylella fastidiosa' ],
    'xft' => [ 'X.fastidiosa_T', 'Xylella fastidiosa Temecula1' ],
    'ype' => [ 'Y.pestis', 'Yersinia pestis' ],
    'ypk' => [ 'Y.pestis_KIM', 'Yersinia pestis KIM' ],
     }

   end

end